@@ -39,16 +39,12 @@ require Tk::Toplevel;
 use strict;
 use vars qw($VERSION $updirImage $folderImage $fileImage);
 
-$VERSION = '3.009'; # $Id: //depot/Tk8/Tk/FBox.pm#9$
+$VERSION = '3.023'; # $Id: //depot/Tk8/Tk/FBox.pm#23 $
 
 use base qw(Tk::Toplevel);
 
 Construct Tk::Widget 'FBox';
 
-my $selectFilePath;
-my $selectFile;
-my $selectPath;
-
 sub import {
     if (defined $_[1] and $_[1] eq 'as_default') {
 	local $^W = 0;
@@ -58,6 +54,12 @@ sub import {
     }
 }
 
+# Note that -sortcmd is experimental and the interface is likely to change.
+# Using -sortcmd is really strange :-(
+# $top->getOpenFile(-sortcmd => sub { package Tk::FBox; uc $b cmp uc $a});
+# or, un-perlish, but useable (now activated in code):
+# $top->getOpenFile(-sortcmd => sub { uc $_[1] cmp uc $_[0]});
+
 sub Populate {
     my($w, $args) = @_;
 
@@ -71,11 +73,12 @@ sub Populate {
     my $f1 = $w->Frame;
     my $lab = $f1->Label(-text => 'Directory:', -underline => 0);
     $w->{'dirMenu'} = my $dirMenu =
-      $f1->Optionmenu(-textvariable => \$w->{'selectPath'},
+      $f1->Optionmenu(-variable => \$w->{'selectPath'},
+		      -textvariable => \$w->{'selectPath'},
 		      -command => ['SetPath', $w]);
     my $upBtn = $f1->Button;
-    if (!defined $updirImage) {
-	$updirImage = $w->Bitmap(-data => <<EOF);
+    if (!defined $updirImage->{$w->MainWindow}) {
+        $updirImage->{$w->MainWindow} = $w->Bitmap(-data => <<EOF);
 #define updir_width 28
 #define updir_height 16
 static char updir_bits[] = {
@@ -87,7 +90,7 @@ static char updir_bits[] = {
    0xf0, 0xff, 0xff, 0x01};
 EOF
     }
-    $upBtn->configure(-image => $updirImage);
+    $upBtn->configure(-image => $updirImage->{$w->MainWindow});
     $dirMenu->configure(-takefocus => 1, -highlightthickness => 2);
     $upBtn->pack(-side => 'right', -padx => 4, -fill => 'both');
     $lab->pack(-side => 'left', -padx => 4, -fill => 'both');
@@ -95,7 +98,8 @@ EOF
 
     $w->{'icons'} = my $icons =
       $w->IconList(-browsecmd => ['ListBrowse', $w],
-		   -command   => ['ListInvoke', $w]);
+		   -command   => ['ListInvoke', $w],
+		  );
 
     # f2: the frame with the OK button and the "file name" field
     my $f2 = $w->Frame(-bd => 0);
@@ -105,7 +109,8 @@ EOF
 
     # The font to use for the icons. The default Canvas font on Unix
     # is just deviant.
-    $w->{'icons'}{'font'} = $ent->cget(-font);
+#    $w->{'icons'}{'font'} = $ent->cget(-font);
+    $w->{'icons'}->configure(-font => $ent->cget(-font));
 
     # f3: the frame with the cancel button and the file types field
     my $f3 = $w->Frame(-bd => 0);
@@ -125,6 +130,7 @@ EOF
        -relief => $f2_lab->cget(-relief),
        -padx => $f2_lab->cget(-padx),
        -pady => $f2_lab->cget(-pady),
+       -takefocus => 0,
       );
     $typeMenuLab->bindtags([$typeMenuLab, 'Label',
 			    $typeMenuLab->toplevel, 'all']);
@@ -185,28 +191,33 @@ EOF
     $w->bind('<Alt-o>',['InvokeBtn','Open']);
     $w->bind('<Alt-s>',['InvokeBtn','Save']);
     $w->protocol('WM_DELETE_WINDOW', ['CancelCmd', $w]);
+    $w->OnDestroy(['CancelCmd', $w]);
 
     # Build the focus group for all the entries
     $w->FG_Create;
     $w->FG_BindIn($ent, ['EntFocusIn', $w]);
     $w->FG_BindOut($ent, ['EntFocusOut', $w]);
 
-    $w->SetPath(Cwd::cwd());
+    $w->SetPath(_cwd());
 
     $w->ConfigSpecs(-defaultextension => ['PASSIVE', undef, undef, undef],
-		    -filetypes        => ['PASSIVE', undef, undef, undef],
-		    -initialdir       => ['PASSIVE', undef, undef, undef],
-		    -initialfile      => ['PASSIVE', undef, undef, undef],
-		    -title            => ['PASSIVE', undef, undef, undef],
-		    -type             => ['PASSIVE', undef, undef, 'open'],
-		    -filter           => ['PASSIVE', undef, undef, '*'],
-		    -force            => ['PASSIVE', undef, undef, 0],
-		   );
-
+                    -filetypes        => ['PASSIVE', undef, undef, undef],
+                    -initialdir       => ['PASSIVE', undef, undef, undef],
+                    -initialfile      => ['PASSIVE', undef, undef, undef],
+#		    -sortcmd          => ['PASSIVE', undef, undef, sub { lc($a) cmp lc($b) }],
+		    -sortcmd          => ['PASSIVE', undef, undef, sub { lc($_[0]) cmp lc($_[1]) }],
+                    -title            => ['PASSIVE', undef, undef, undef],
+                    -type             => ['PASSIVE', undef, undef, 'open'],
+                    -filter           => ['PASSIVE', undef, undef, '*'],
+                    -force            => ['PASSIVE', undef, undef, 0],
+                    'DEFAULT'         => [$icons],
+                   );
+    # So-far-failed attempt to break reference loops ...
+    $w->_OnDestroy(qw(dirMenu icons typeMenuLab typeMenuBtn okBtn ent updateId));
     $w;
 }
 
-
+# -initialdir fix with ResolveFile
 sub Show {
     my $w = shift;
 
@@ -219,8 +230,9 @@ sub Show {
     {
 	my $initialdir = $w->cget(-initialdir);
 	if (defined $initialdir) {
-	    if (-d $initialdir) {
-		$w->{'selectPath'} = $initialdir;
+	    my ($flag, $path, $file) = ResolveFile($initialdir, 'junk');
+	    if ($flag eq 'OK' or $flag eq 'FILE') {
+		$w->{'selectPath'} = $path;
 	    } else {
 		$w->Error("\"$initialdir\" is not a valid directory");
 	    }
@@ -247,7 +259,7 @@ sub Show {
 	$typeMenuBtn->configure(-state => 'normal');
 	$typeMenuLab->configure(-state => 'normal');
     } else {
-	$w->configure(-filter => '*');
+#XXX	$w->configure(-filter => '*');
 	$typeMenuBtn->configure(-state => 'disabled',
 				-takefocus => 0);
 	$typeMenuLab->configure(-state => 'disabled');
@@ -259,9 +271,9 @@ sub Show {
     # display and de-iconify it.
     $w->withdraw;
     $w->idletasks;
-    my $x = $w->screenwidth / 2 - $w->reqwidth / 2 - $w->parent->vrootx;
-    my $y = $w->screenheight / 2 - $w->reqheight / 2 - $w->parent->vrooty;
-    $w->geometry($w->reqwidth .'x'. $w->reqheight);
+    my $x = int($w->screenwidth / 2 - $w->reqwidth / 2 - $w->parent->vrootx);
+    my $y = int($w->screenheight / 2 - $w->reqheight / 2 - $w->parent->vrooty);
+    $w->geometry("+$x+$y");
 
     {
 	my $title = $w->cget(-title);
@@ -280,30 +292,34 @@ sub Show {
     my $ent = $w->{'ent'};
     $ent->focus;
     $ent->delete(0, 'end');
-    $ent->insert(0, $w->{'selectFile'});
-    $ent->selectionFrom(0);
-    $ent->selectionTo('end');
-    $ent->icursor('end');
+    if (defined $w->{'selectFile'} && $w->{'selectFile'} ne '') {
+	$ent->insert(0, $w->{'selectFile'});
+	$ent->selectionFrom(0);
+	$ent->selectionTo('end');
+	$ent->icursor('end');
+    }
 
     # 8. Wait for the user to respond, then restore the focus and
     # return the index of the selected button.  Restore the focus
     # before deleting the window, since otherwise the window manager
     # may take the focus away so we can't redirect it.  Finally,
     # restore any grab that was in effect.
-    $w->waitVariable(\$selectFilePath);
+    $w->waitVariable(\$w->{'selectFilePath'});
     eval {
 	$oldFocus->focus if $oldFocus;
     };
-    $w->grabRelease;
-    $w->withdraw;
-    if ($oldGrab) {
+    if (Tk::Exists($w)) { # widget still exists
+	$w->grabRelease;
+	$w->withdraw;
+    }
+    if (Tk::Exists($oldGrab) && $oldGrab->viewable) {
 	if ($grabStatus eq 'global') {
 	    $oldGrab->grabGlobal;
 	} else {
 	    $oldGrab->grab;
 	}
     }
-    return $selectFilePath;
+    return $w->{'selectFilePath'};
 }
 
 # tkFDialog_UpdateWhenIdle --
@@ -339,15 +355,15 @@ sub Update {
     } else {
 	delete $w->{'updateId'};
     }
-    unless (defined $folderImage) {
+    unless (defined $folderImage->{$w->MainWindow}) {
 	require Tk::Pixmap;
-	$folderImage = $w->Pixmap(-file => Tk->findINC('folder.xpm'));
-	$fileImage   = $w->Pixmap(-file => Tk->findINC('file.xpm'));
+	$folderImage->{$w->MainWindow} = $w->Pixmap(-file => Tk->findINC('folder.xpm'));
+	$fileImage->{$w->MainWindow}   = $w->Pixmap(-file => Tk->findINC('file.xpm'));
     }
-    my $folder = $folderImage;
-    my $file   = $fileImage;
-    my $appPWD = Cwd::cwd();
-    if (!chdir $w->{'selectPath'}) {
+    my $folder = $folderImage->{$w->MainWindow};
+    my $file   = $fileImage->{$w->MainWindow};
+    my $appPWD = _cwd();
+    if (!ext_chdir($w->{'selectPath'})) {
 	# We cannot change directory to $data(selectPath). $data(selectPath)
 	# should have been checked before tkFDialog_Update is called, so
 	# we normally won't come to here. Anyways, give an error and abort
@@ -357,7 +373,7 @@ sub Update {
 		       $w->{'selectPath'} . "\".\nPermission denied.",
 		       -icon => 'warning',
 		      );
-	chdir $appPWD;
+	ext_chdir($appPWD);
 	return;
     }
 
@@ -372,31 +388,39 @@ sub Update {
     my $icons = $w->{'icons'};
     $icons->DeleteAll;
 
-    # Make the dir list
-    my %hasDoneDir;
-    foreach my $f (sort { lc($a) cmp lc($b) } glob('.* *')) {
-	next if $f eq '.' or $f eq '..';
-	if (-d "./$f" and not exists $hasDoneDir{$f}) {
-	    $icons->Add($folder, $f);
-	    $hasDoneDir{$f}++;
+    # Make the dir & file list
+    my $cwd = _cwd();
+    local *FDIR;
+    if (opendir(FDIR, $cwd)) {
+        my @files;
+#	my $sortcmd = $w->cget(-sortcmd);
+	my $sortcmd = sub { $w->cget(-sortcmd)->($a,$b) };
+	my $flt = $w->cget(-filter);
+	my $fltcb;
+	if (ref $flt eq 'CODE') {
+	    $fltcb = $flt;
+	} else {
+	    $flt = _rx_to_glob($flt);
+	}
+        foreach my $f (sort $sortcmd readdir(FDIR)) {
+            next if $f eq '.' or $f eq '..';
+	    if ($fltcb) {
+		next if !$fltcb->($w, $f, $cwd);
+	    } else {
+		next if -f $f && $f !~ m!$flt!;
+	    }
+            if (-d $f) {
+		$icons->Add($folder, $f);
+	    } else {
+		push @files, $f;
+	    }
+	}
+	closedir(FDIR);
+	foreach my $f (@files) {
+	    $icons->Add($file, $f);
 	}
     }
 
-    # Make the file list
-    my @files;
-    if ($w->cget(-filter) eq '*') {
-	@files = sort { lc($a) cmp lc($b) } glob('.* *');
-    } else {
-	@files = sort { lc($a) cmp lc($b) } glob($w->cget(-filter));
-    }
-    my $top = 0;
-    my %hasDoneFile;
-    foreach my $f (@files) {
-	if (-f "./$f" and not exists $hasDoneFile{$f}) {
-	    $icons->Add($file, $f);
-	    $hasDoneFile{$f}++;
-	}
-    }
     $icons->Arrange;
 
     # Update the Directory: option menu
@@ -407,13 +431,15 @@ sub Update {
 	push @list, $dir;
     }
     my $dirMenu = $w->{'dirMenu'};
-    $dirMenu->options([]);
-    my $var = $w->{'selectPath'};
-    $dirMenu->addOptions(@list);
-    $w->{'selectPath'} = $var; # workaround
+    $dirMenu->configure(-options => \@list);
 
     # Restore the PWD to the application's PWD
-    chdir $appPWD;
+    ext_chdir($appPWD);
+
+    # Restore the Save label
+    if ($w->cget(-type) eq 'save') {
+	$w->{'okBtn'}->configure(-text => 'Save');
+    }
 
     # turn off the busy cursor.
     $ent->configure(-cursor => $entCursor);
@@ -483,7 +509,7 @@ sub SetFilter {
 #
 sub ResolveFile {
     my($context, $text, $defaultext) = @_;
-    my $appPWD = Cwd::cwd();
+    my $appPWD = _cwd();
     my $path = JoinFile($context, $text);
     $path = "$path$defaultext" if ($path !~ /\..+$/) and defined $defaultext;
     # Cannot just test for existance here as non-existing files are
@@ -492,37 +518,37 @@ sub ResolveFile {
     my($directory, $file, $flag);
     if (-e $path) {
 	if (-d $path) {
-	    if (!chdir $path) {
+	    if (!ext_chdir($path)) {
 		return ('CHDIR', $path, '');
 	    }
-	    $directory = Cwd::cwd();
+	    $directory = _cwd();
 	    $file = '';
 	    $flag = 'OK';
-	    chdir $appPWD;
+	    ext_chdir($appPWD);
 	} else {
 	    my $dirname = File::Basename::dirname($path);
-	    if (!chdir $dirname) {
+	    if (!ext_chdir($dirname)) {
 		return ('CHDIR', $dirname, '');
 	    }
-	    $directory = Cwd::cwd();
+	    $directory = _cwd();
 	    $file = File::Basename::basename($path);
 	    $flag = 'OK';
-	    chdir $appPWD;
+	    ext_chdir($appPWD);
 	}
     } else {
 	my $dirname = File::Basename::dirname($path);
 	if (-e $dirname) {
-	    if (!chdir $dirname) {
+	    if (!ext_chdir($dirname)) {
 		return ('CHDIR', $dirname, '');
 	    }
-	    $directory = Cwd::cwd();
+	    $directory = _cwd();
 	    $file = File::Basename::basename($path);
 	    if ($file =~ /[*?]/) {
 		$flag = 'PATTERN';
 	    } else {
 		$flag = 'FILE';
 	    }
-	    chdir $appPWD;
+	    ext_chdir($appPWD);
 	} else {
 	    $directory = $dirname;
 	    $file = File::Basename::basename($path);
@@ -587,7 +613,7 @@ sub ActivateEnt {
 	if ($w->cget(-type) eq 'open') {
 	    $w->messageBox(-icon => 'warning',
 			   -type => 'OK',
-			   -message => 'File \"' . TclFileJoin($path, $file)
+			   -message => 'File "' . TclFileJoin($path, $file)
 			   . '" does not exist.');
 	    $ent->selection('from', 0);
 	    $ent->selection('to', 'end');
@@ -641,7 +667,7 @@ sub UpDirCmd {
 # if the filename begins with ~
 sub JoinFile {
     my($path, $file) = @_;
-    if ($file =~ /^~/) {
+    if ($file =~ /^~/ && -e "$path/$file") {
 	TclFileJoin($path, "./$file");
     } else {
 	TclFileJoin($path, $file);
@@ -654,8 +680,21 @@ sub TclFileJoin {
     foreach (@_) {
 	if (m|^/|) {
 	    $path = $_;
+	}
+	elsif (m|^[a-z]:/|i) {  # DOS-ish
+	    $path = $_;
+	} elsif ($_ eq '~') {
+	    $path = _get_homedir();
+	} elsif (m|^~/(.*)|) {
+	    $path = _get_homedir() . "/" . $1;
 	} elsif (m|^~([^/]+)(.*)|) {
-	    $path = (eval { (getpwnam($1))[7] } || $ENV{'HOME'} || '') . $2;
+	    my($user, $p) = ($1, $2);
+	    my $dir = _get_homedir($user);
+	    if (!defined $dir) {
+		$path = "~$user$p";
+	    } else {
+		$path = $dir . $p;
+	    }
 	} elsif ($path eq '/' or $path eq '') {
 	    $path .= $_;
 	} else {
@@ -696,7 +735,7 @@ sub OkCmd {
 #
 sub CancelCmd {
     my $w = shift;
-    undef $selectFilePath;
+    undef $w->{'selectFilePath'};
 }
 
 # Gets called when user browses the IconList widget (dragging mouse, arrow
@@ -729,18 +768,25 @@ sub ListInvoke {
     return if ($text eq '');
     my $file = JoinFile($w->{'selectPath'}, $text);
     if (-d $file) {
-	my $appPWD = Cwd::cwd();
-	if (!chdir $file) {
+	my $appPWD = _cwd();
+	if (!ext_chdir($file)) {
 	    $w->messageBox(-type => 'OK',
 			   -message => "Cannot change to the directory \"$file\".\nPermission denied.",
 			   -icon => 'warning');
 	} else {
-	    chdir $appPWD;
+	    ext_chdir($appPWD);
 	    $w->SetPath($file);
 	}
     } else {
-	$w->{'selectFile'} = $file;
-	$w->Done;
+        my($flag, $path, $file) = ResolveFile($w->{'selectPath'}, $text);
+        if ($flag ne 'OK') {
+            $w->messageBox(-type => 'OK',
+                           -message => "Cannot resolve $w->{'selectPath'}/$text.",
+                           -icon => 'error');
+        } else {
+            $path = JoinFile($path, $file);
+            $w->Done($path);
+        }
     }
 }
 
@@ -767,7 +813,7 @@ sub Done {
 	    return unless (lc($reply) eq 'yes');
 	}
     }
-    $selectFilePath = ($_selectFilePath ne '' ? $_selectFilePath : undef);
+    $w->{'selectFilePath'} = ($_selectFilePath ne '' ? $_selectFilePath : undef);
 }
 
 sub FDialog {
@@ -827,5 +873,64 @@ sub GetFileTypes {
     return @types;
 }
 
+# ext_chdir --
+#
+#       Change directory with tilde substitution
+#
+sub ext_chdir {
+    my $dir = shift;
+    if ($dir eq '~') {
+	chdir _get_homedir();
+    } elsif ($dir =~ m|^~/(.*)|) {
+	chdir _get_homedir() . "/" . $1;
+    } elsif ($dir =~ m|^~([^/]+(.*))|) {
+	chdir _get_homedir($1) . $2;
+    } else {
+	chdir $dir;
+    }
+}
+
+# _get_homedir --
+#
+#       Get home directory of the current user
+#
+sub _get_homedir {
+    my($user) = @_;
+    if (!defined $user) {
+	eval {
+	    local $SIG{__DIE__};
+	    (getpwuid($<))[7];
+	} || $ENV{HOME} || undef; # chdir undef changes to home directory, too
+    } else {
+	eval {
+	    local $SIG{__DIE__};
+	    (getpwnam($user))[7];
+	};
+    }
+}
+
+sub _cwd {
+    #Cwd::cwd();
+    Cwd::fastcwd(); # this is taint-safe
+}
+
+sub _untaint {
+    my $s = shift;
+    $s =~ /^(.*)$/;
+    $1;
+}
+
+sub _rx_to_glob {
+    my $arg = shift;
+    $arg = join('|', split(' ', $arg));
+    $arg =~ s!([\.\+])!\\$1!g;
+    $arg =~ s!\*!.*!g;
+    $arg = "^" . $arg . "\$";
+    if ($] >= 5.005) {
+	$arg = qr($arg);
+    }
+    $arg;
+}
+
 1;
 
