@@ -1,4 +1,4 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::MainWindow;
@@ -8,7 +8,7 @@ BEGIN { @MainWindow::ISA = 'Tk::MainWindow' }
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.039'; # $Id: //depot/Tk8/Tk/MainWindow.pm#39$
+$VERSION = '3.049'; # $Id: //depot/Tk8/Tk/MainWindow.pm#49 $
 
 use Tk::CmdLine;
 use Tk qw(catch);
@@ -20,12 +20,12 @@ $| = 1;
 
 my $pid = $$;
 
-my @Windows = ();
+my %Windows = ();
 
 sub CreateArgs
 {
  my ($class,$args) = @_;
- my $cmd = Tk::CmdLine::CreateArgs();
+ my $cmd = Tk::CmdLine->CreateArgs();
  my $key;
  foreach $key (keys %$cmd)
   {
@@ -66,10 +66,17 @@ sub new
 	 $top->geometry($geometry);
      }
  }
- push(@Windows,$top);
+ $Windows{$top} = $top;
  return $top;
 }
 
+sub _Destroyed
+{
+ my $top = shift;
+ $top->SUPER::_Destroyed;
+ delete $Windows{$top};
+}
+
 sub InitBindings
 {
  my $mw = shift;
@@ -82,7 +89,8 @@ sub InitBindings
    $mw->eventAdd(qw[<<Cut>> <Control-Key-x> <Key-F20> <Meta-Key-w>]);
    $mw->eventAdd(qw[<<Copy>> <Control-Key-c> <Key-F16> <Control-Key-w>]);
    $mw->eventAdd(qw[<<Paste>> <Control-Key-v> <Key-F18> <Control-Key-y>]);
-   $mw->eventAdd(qw[<<Undo>> <Control-Key-z> <Key-Undo> <Key-F14>]);
+   $mw->eventAdd(qw[<<Undo>> <Control-Key-z> <Key-Undo> <Key-F14>
+                    <Control-Key-underscore>]);
    $mw->eventAdd(qw[<<Redo>> <Control-Key-y> <Shift-Key-Undo> <Key-F12> <Shift-Key-F14>]);
   }
  else
@@ -102,15 +110,27 @@ sub InitBindings
 
 sub Existing
 {
- grep( Tk::Exists($_), @Windows);
+ my @Windows;
+ foreach my $name (keys %Windows)
+  {
+   my $obj = $Windows{$name};
+   if (Tk::Exists($obj))
+    {
+     push(@Windows,$obj);
+    }
+   else
+    {
+     delete $Windows{$name};
+    }
+  }
+ return @Windows;
 }
 
 END
 {
  if ($pid == $$)
   {
-   my $top;
-   while ($top = pop(@Windows))
+   foreach my $top (values %Windows)
     {
      if ($top->IsWidget)
       {
@@ -124,7 +144,7 @@ END
   }
 }
 
-sub CmdLine { return shift }
+sub CmdLine { return shift->command }
 
 sub WMSaveYourself
 {
