@@ -1,4 +1,4 @@
-/* 
+/*
  * tkImage.c --
  *
  *	This file contains code that allows images to be
@@ -62,7 +62,7 @@ static void		EmbImageDisplayProc _ANSI_ARGS_((
 static int		EmbImageLayoutProc _ANSI_ARGS_((TkText *textPtr,
 			    TkTextIndex *indexPtr, TkTextSegment *segPtr,
 			    int offset, int maxX, int maxChars,
-			    int noCharsYet, Tk_Uid wrapMode,
+			    int noCharsYet, TkWrapMode wrapMode,
 			    TkTextDispChunk *chunkPtr));
 static void		EmbImageProc _ANSI_ARGS_((ClientData clientData,
 			    int x, int y, int width, int height,
@@ -217,7 +217,7 @@ TkTextImageCmd(textPtr, interp, argc, argv)
 	/*
 	 * Don't allow insertions on the last (dummy) line of the text.
 	 */
-    
+
 	lineIndex = TkBTreeLineIndex(index.linePtr);
 	if (lineIndex == TkBTreeNumLines(textPtr->tree)) {
 	    lineIndex--;
@@ -347,7 +347,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
     	return TCL_OK;
     }
 
-    /* 
+    /*
      * Find a unique name for this image.  Use imageName (or imageString)
      * if available, otherwise tack on a #nn and use it.  If a name is already
      * associated with this image, delete the name.
@@ -374,7 +374,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
 	    if (new > count) {
 		count = new;
 	    }
-	    if (len == (int) strlen(haveName)) {
+	    if (len == strlen(haveName)) {
 	    	conflict = 1;
 	    }
 	}
@@ -478,8 +478,8 @@ AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
 					 * information about how to reclaim
 					 * storage for return string. */
-{   
-    Arg result = NULL; 
+{
+    Arg result = NULL;
     *freeProcPtr = (Tcl_FreeProc *) NULL;
     switch (((TkTextEmbImage *) widgRec)->align) {
 	case ALIGN_BASELINE:
@@ -612,8 +612,8 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
 				 * many characters. */
     int noCharsYet;		/* Non-zero means no characters have been
 				 * assigned to this line yet. */
-    Tk_Uid wrapMode;		/* Wrap mode to use for line: tkTextCharUid,
-				 * tkTextNoneUid, or tkTextWordUid. */
+    TkWrapMode wrapMode;	/* Wrap mode to use for line: TEXT_WRAPMODE_CHAR,
+				 * TEXT_WRAPMODE_NONE, or TEXT_WRAPMODE_WORD. */
     register TkTextDispChunk *chunkPtr;
 				/* Structure to fill in with information
 				 * about this chunk.  The x field has already
@@ -638,7 +638,7 @@ EmbImageLayoutProc(textPtr, indexPtr, eiPtr, offset, maxX, maxChars,
 	height += 2*eiPtr->body.ei.padY;
     }
     if ((width > (maxX - chunkPtr->x))
-	    && !noCharsYet && (textPtr->wrapMode != tkTextNoneUid)) {
+	    && !noCharsYet && (textPtr->wrapMode != TEXT_WRAPMODE_NONE)) {
 	return 0;
     }
 
