@@ -1,7 +1,7 @@
-/* 
+/*
  * tkUnixSelect.c --
  *
- *	This file contains X specific routines for manipulating 
+ *	This file contains X specific routines for manipulating
  *	selections.
  *
  * Copyright (c) 1995 Sun Microsystems, Inc.
@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkUnixSelect.c 1.5 96/03/29 14:14:31
+ * RCS: @(#) $Id: tkUnixSelect.c,v 1.2 1998/09/14 18:23:57 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -87,7 +87,7 @@ static void		SelRcvIncrProc _ANSI_ARGS_((ClientData clientData,
 static void		SelTimeoutProc _ANSI_ARGS_((ClientData clientData));
 static void		FreeHandler _ANSI_ARGS_((ClientData clientData));
 static int		HandleCompat _ANSI_ARGS_((ClientData clientData,
-			    int offset, long *buffer, int maxBytes, 
+			    int offset, long *buffer, int maxBytes,
 			    Atom type, Tk_Window tkwin));
 
 /*
@@ -224,7 +224,8 @@ TkSelPropProc(eventPtr)
     register XEvent *eventPtr;		/* X PropertyChange event. */
 {
     register IncrInfo *incrPtr;
-    int i, format = 8;
+    unsigned i;
+    int format = 8;
     Atom target, formatType = None;
     register TkSelHandler *selPtr;
     long buffer[TK_SEL_WORDS_AT_ONCE];
@@ -459,7 +460,7 @@ TkSelEventProc(tkwin, eventPtr)
 	ConvertSelection(winPtr, &eventPtr->xselectionrequest);
 	return;
     }
-}                        
+}
 
 /*
  *----------------------------------------------------------------------
@@ -551,7 +552,7 @@ ConvertSelection(winPtr, eventPtr)
     IncrInfo incr;			/* State of selection conversion. */
     Atom singleInfo[2];			/* incr.multAtoms points here except
 					 * for multiple conversions. */
-    int i;
+    unsigned i;
     Tk_ErrorHandler errorHandler;
     TkSelectionInfo *infoPtr;
     TkSelInProgress ip;
@@ -592,6 +593,8 @@ ConvertSelection(winPtr, eventPtr)
      * conversions to perform.
      */
 
+
+
     incr.winPtr = winPtr;
     incr.selection = eventPtr->selection;
     if (eventPtr->target != winPtr->dispPtr->multipleAtom) {
@@ -615,6 +618,14 @@ ConvertSelection(winPtr, eventPtr)
 		0, MAX_PROP_WORDS, False, XA_ATOM,
 		&type, &format, &incr.numConversions, &bytesAfter,
 		(unsigned char **) &incr.multAtoms);
+	if (result == Success && incr.numConversions == 0 && format == 32 &&
+            type != XA_ATOM && type != None) {
+	    result = XGetWindowProperty(eventPtr->display,
+		eventPtr->requestor, eventPtr->property,
+		0, MAX_PROP_WORDS, False, type,
+		&type, &format, &incr.numConversions, &bytesAfter,
+		(unsigned char **) &incr.multAtoms);
+	}
 	if ((result != Success) || (bytesAfter != 0) || (format != 32)
 		|| (type == None)) {
 	    if (incr.multAtoms != NULL) {
